from zeep.helpers import serialize_object
from zeep.xsd.valueobjects import CompoundValue


def convert_zeep_object(obj):
    if isinstance(obj, CompoundValue):
        obj = serialize_object(obj)

    if isinstance(obj, list):
        return [convert_zeep_object(sub) for sub in obj]

    if isinstance(obj, dict):
        return {k: convert_zeep_object(v) for k, v in obj.items()}

    return obj



from typing import Optional, List

from pip_middleware_fluig.repositories.soap import ClientFluig
from pip_middleware_fluig.interface.ecm_dataset import DatasetQuery, StructConstraint
from pip_middleware_fluig.utils.common_functions import convert_zeep_object


class ECMDatasetService:
    def __init__(self, client: ClientFluig = None) -> None:
        self._client = client or ClientFluig()

    def constraints(self, constraints: List[StructConstraint]):
        constraint_array_element = self._client.get_element_in_xml(element="ns0:searchConstraintDtoArray")
        constraint_element = self._client.get_element_in_xml(element="ns0:searchConstraintDto")
        items = []
        for value in constraints:
            items.append(
                constraint_element(
                    value.constraint_type,
                    value.field_name,
                    value.init_value,
                    value.final_value,
                    value.like_search,
                )
            )
        return constraint_array_element(items)

    def execute_query_dataset(self, dataset: Optional[DatasetQuery]) -> List[dict]:
        query_result = self._client.soap_intance.service.getDataset(
            dataset.company_id,
            dataset.user_name,
            dataset.password,
            dataset.dataset_name,
            dataset.fields,
            self.constraints(constraints=dataset.constraints),
            dataset.order,
        )

        response = convert_zeep_object(query_result)

        serialize_result = []
        items = {}
        for value in response["values"]:
            for k, v in zip(response["columns"], value["value"]):
                items.update({k: v})

            serialize_result.append(items)
        return serialize_result



from typing import Optional
from zeep.client import Client, Settings


class ClientFluig:
    def __init__(
        self, client_soap: Optional[Client] = None, settings: Optional[Settings] = None, *, wsdl_url: str
    ) -> None:
        self._URL_WSDL = wsdl_url
        self._client_soap = client_soap
        self._settings = settings or Settings(strict=False)

    @property
    def soap_intance(self) -> Client:
        if self._client_soap is None:
            self._client_soap = Client(wsdl=self._URL_WSDL, settings=self._settings)
        return self._client_soap

    def get_element_in_xml(self, element: str):
        client = self.soap_intance
        return client.get_type(element)


from typing import List
from pip_middleware_fluig.domain.ecm_services import ECMDatasetService
from pip_middleware_fluig.interface.ecm_dataset import DatasetQueryParams


class FluigECMService:
    def __init__(self) -> None:
        self._domain_ecm_service = ECMDatasetService()

    def get_dataset(self, dataset_params: dict) -> List[dict]:

        params = DatasetQueryParams(**dataset_params)
        return self._domain_ecm_service.execute_query_dataset(params)



from typing import Optional, List
from pydantic import BaseModel, Field


class StructConstraint(BaseModel):
    constraint_type: Optional[str]
    field_name: Optional[str]
    init_value: Optional[str]
    final_value: Optional[str]
    like_search: Optional[str] = ""


class Constraints(BaseModel):
    constraint: List[StructConstraint] = Field(default_factory=list)


class StructField(BaseModel):
    item: Optional[str]


class StructOrder(BaseModel):
    item: Optional[str]


class DatasetQuery(BaseModel):
    company_id: Optional[str]
    user_name: Optional[str]
    password: Optional[str]
    dataset_name: Optional[str]
    fields: List[StructField] = Field(default_factory=list)
    constraints: List[StructConstraint] = Field(default_factory=list)
    order: List[StructOrder] = Field(default_factory=list)


class DatasetQueryParams(BaseModel):
    field_name: Optional[str]
    value: Optional[str]



from typing import Optional
from pydantic import BaseSettings, Field
from typing import Optional

_settings = None


class Settings(BaseSettings):
    environment: Optional[str] = Field(env="ENVIRONMENT")

    @classmethod
    def get_setting(cls):

        global _settings
        if _settings is None:
            _settings = Settings()

        return _settings



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['pip_middleware_fluig',
 'pip_middleware_fluig.domain',
 'pip_middleware_fluig.interface',
 'pip_middleware_fluig.repositories',
 'pip_middleware_fluig.services',
 'pip_middleware_fluig.tests',
 'pip_middleware_fluig.utils']

package_data = \
{'': ['*']}

install_requires = \
['black>=22.1.0,<23.0.0',
 'httpx>=0.22.0,<0.23.0',
 'ipython>=8.2.0,<9.0.0',
 'pydantic>=1.9.0,<2.0.0',
 'pytest>=7.1.1,<8.0.0',
 'safety>=1.10.3,<2.0.0',
 'twine>=3.8.0,<4.0.0',
 'zeep>=4.1.0,<5.0.0']

setup_kwargs = {
    'name': 'pip-middleware-fluig',
    'version': '0.1.5',
    'description': 'middleware service fluig plataform',
    'long_description': None,
    'author': 'Rodrigo Becker',
    'author_email': 'rodrigo.beckermore@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
